#!/usr/bin/python

"""Definitions for physical constants
"""

R_d = 287.0 # J K^-1 kg^-1; Wallace and Hobbs Eq. 3.11
R_v = 461.51 # J K^-1 kg^-1; Wallace and Hobbs Eq. 3.13
h = 6.62617e-34  # J s Planck
k = 1.380662e-23 # J/K Boltzmann
c = 2.99792458e8  # m/s speed of light

